#
#    Copyright (c) 2018 - 2020 Analog Flavor EURL
#
#    Contact us for support:
#        http://www.analogflavor.com
#
# -----------------------------------------------------------------------------
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#
#    * Redistributions of source code must retain the above copyright notice,
#      this list of conditions and the following disclaimer.
#
#    * Redistributions in binary form must reproduce the above copyright
#      notice, this list of conditions and the following disclaimer in the
#      documentation and/or other materials provided with the distribution.
#
#    * Neither the name of Analog Flavor EURL nor 
#      the names of its contributors may be used to endorse or promote products derived from
#      this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
# LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
# CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
# SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
# INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
# CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
# ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
# POSSIBILITY OF SUCH DAMAGE.
# -----------------------------------------------------------------------------
#


## \file 0005-high-level-channel-functions.py
#  \brief This file contains an example for the use of the WienerMpodLvHvSnmpPython module.
#
#  The used functions allow to control and monitor a single Wiener MPOD crate over SNMP.
#
#  The comments in the example code give more explanations on the performed API function calls.

import sys
import time
import math
import argparse
import import_mpod_snmp_module

def test_channel_functions(mpod_channel) :

    # command: getConfigMaxCurrent
    #     input: -
    #     output: real value
    result = mpod_channel.getConfigMaxCurrent()
    print("getConfigMaxCurrent() returns \"{0}\"".format(result))

    # command: getConfigMaxSenseVoltage
    #     input: -
    #     output: real value
    result = mpod_channel.getConfigMaxSenseVoltage()
    print("getConfigMaxSenseVoltage() returns \"{0}\"".format(result))

    # command: getConfigMaxTerminalVoltage
    #     input: -
    #     output: real value
    result = mpod_channel.getConfigMaxTerminalVoltage()
    print("getConfigMaxTerminalVoltage() returns \"{0}\"".format(result))

    # command: getCurrent
    #     input: -
    #     output: real value
    result = mpod_channel.getCurrent()
    print("getCurrent() returns \"{0}\"".format(result))

    # command: getCurrentFallRate
    #     input: -
    #     output: real value
    result = mpod_channel.getCurrentFallRate()
    print("getCurrentFallRate() returns \"{0}\"".format(result))

    # command: getCurrentRiseRate
    #     input: -
    #     output: real value
    result = mpod_channel.getCurrentRiseRate()
    print("getCurrentRiseRate() returns \"{0}\"".format(result))

    # command: getIndex
    #     input: -
    #     output: integer value
    result = mpod_channel.getIndex()
    print("getIndex() returns \"{0}\"".format(result))

    # command: getMeasurementCurrent
    #     input: -
    #     output: real value
    result = mpod_channel.getMeasurementCurrent()
    print("getMeasurementCurrent() returns \"{0}\"".format(result))

    # command: getMeasurementSenseVoltage
    #     input: -
    #     output: real value
    result = mpod_channel.getMeasurementSenseVoltage()
    print("getMeasurementSenseVoltage() returns \"{0}\"".format(result))

    # command: getMeasurementTemperature
    #     input: -
    #     output: real value
    result = mpod_channel.getMeasurementTemperature()
    print("getMeasurementTemperature() returns \"{0}\"".format(result))

    # command: getMeasurementTerminalVoltage
    #     input: -
    #     output: real value
    result = mpod_channel.getMeasurementTerminalVoltage()
    print("getMeasurementTerminalVoltage() returns \"{0}\"".format(result))

    # command: getStatus
    #     input: -
    #     output: integer value
    result = mpod_channel.getStatus()
    print("getStatus() returns \"{0}\"".format(result))

    # command: getStatusAsString
    #     input: -
    #     output: string value
    result = mpod_channel.getStatusAsString()
    print("getStatusAsString() returns \"{0}\"".format(result))

    # command: getSupervisionBehavior
    #     input: -
    #     output: integer value
    result = mpod_channel.getSupervisionBehavior()
    print("getSupervisionBehavior() returns \"{0}\"".format(result))

    # command: getSupervisionMaxCurrent
    #     input: -
    #     output: real value
    result = mpod_channel.getSupervisionMaxCurrent()
    print("getSupervisionMaxCurrent() returns \"{0}\"".format(result))

    # command: getSupervisionMaxPower
    #     input: -
    #     output: real value
    result = mpod_channel.getSupervisionMaxPower()
    print("getSupervisionMaxPower() returns \"{0}\"".format(result))

    # command: getSupervisionMaxTerminalVoltage
    #     input: -
    #     output: real value
    result = mpod_channel.getSupervisionMaxTerminalVoltage()
    print("getSupervisionMaxTerminalVoltage() returns \"{0}\"".format(result))

    # command: getSupervisionMinSenseVoltage
    #     input: -
    #     output: real value
    result = mpod_channel.getSupervisionMinSenseVoltage()
    print("getSupervisionMinSenseVoltage() returns \"{0}\"".format(result))

    # command: getSwitch
    #     input: -
    #     output: integer value
    result = mpod_channel.getSwitch()
    print("getSwitch() returns \"{0}\"".format(result))

    # command: getTripTime
    #     input: -
    #     output: integer value
    result = mpod_channel.getTripTime()
    print("getTripTime() returns \"{0}\"".format(result))

    # command: getUserConfig
    #     input: -
    #     output: integer value
    result = mpod_channel.getUserConfig()
    print("getUserConfig() returns \"{0}\"".format(result))

    # command: getVoltage
    #     input: -
    #     output: real value
    result = mpod_channel.getVoltage()
    print("getVoltage() returns \"{0}\"".format(result))

    # command: getVoltageFallRate
    #     input: -
    #     output: real value
    result = mpod_channel.getVoltageFallRate()
    print("getVoltageFallRate() returns \"{0}\"".format(result))

    # command: getVoltageRiseRate
    #     input: -
    #     output: real value
    result = mpod_channel.getVoltageRiseRate()
    print("getVoltageRiseRate() returns \"{0}\"".format(result))

    # command: setCurrent
    #     input: Value
    #     output: True if sucessfull
    val = 0.0005
    result = mpod_channel.setCurrent(val)
    print("setCurrent({0}) returns \"{1}\"".format(val, result))

    # command: setCurrentFallRate
    #     input: Value
    #     output: True if sucessfull
    val = 1.1
    result = mpod_channel.setCurrentFallRate(val)
    print("setCurrentFallRate({0}) returns \"{1}\"".format(val, result))

    # command: setCurrentRiseRate
    #     input: Value
    #     output: True if sucessfull
    val = 1.2
    result = mpod_channel.setCurrentRiseRate(val)
    print("setCurrentRiseRate({0}) returns \"{1}\"".format(val, result))

    # command: setSupervisionBehavior
    #     input: Value
    #     output: True if sucessfull
    val = 17680
    result = mpod_channel.setSupervisionBehavior(val)
    print("setSupervisionBehavior({0}) returns \"{1}\"".format(val, result))

    # command: setSupervisionMaxCurrent
    #     input: Value
    #     output: True if sucessfull
    val = 1e-3
    result = mpod_channel.setSupervisionMaxCurrent(val)
    print("setSupervisionMaxCurrent({0}) returns \"{1}\"".format(val, result))

    # command: setSupervisionMaxTerminalVoltage
    #     input: Value
    #     output: True if sucessfull
    val = 3.0
    result = mpod_channel.setSupervisionMaxTerminalVoltage(val)
    print("setSupervisionMaxTerminalVoltage({0}) returns \"{1}\"".format(val, result))

    # command: setSupervisionMinSenseVoltage
    #     input: Value
    #     output: True if sucessfull
    val = 1.0
    result = mpod_channel.setSupervisionMinSenseVoltage(val)
    print("setSupervisionMinSenseVoltage({0}) returns \"{1}\"".format(val, result))

    # command: setSwitch
    #     input: Value
    #     output: True if sucessfull
    val = 1
    result = mpod_channel.setSwitch(val)
    print("setSwitch({0}) returns \"{1}\"".format(val, result))

    # command: setTripTime
    #     input: Value
    #     output: True if sucessfull
    val = 100
    result = mpod_channel.setTripTime(val)
    print("setTripTime({0}) returns \"{1}\"".format(val, result))

    # command: setUserConfig
    #     input: Value
    #     output: True if sucessfull
    val = 8
    result = mpod_channel.setUserConfig(val)
    print("setUserConfig({0}) returns \"{1}\"".format(val, result))

    # command: setVoltage
    #     input: Value
    #     output: True if sucessfull
    val = 4.5
    result = mpod_channel.setVoltage(val)
    print("setVoltage({0}) returns \"{1}\"".format(val, result))

    # command: setVoltageFallRate
    #     input: Value
    #     output: True if sucessfull
    val = 1.0
    result = mpod_channel.setVoltageFallRate(val)
    print("setVoltageFallRate({0}) returns \"{1}\"".format(val, result))

    # command: setVoltageRiseRate
    #     input: Value
    #     output: True if sucessfull
    val = 2.0
    result = mpod_channel.setVoltageRiseRate(val)
    print("setVoltageRiseRate({0}) returns \"{1}\"".format(val, result))

    ## it takes some time until the the new value is taken into account (write and read to device)
    print("... waiting before comparing read and written value ...")
    time.sleep(wait_delay)

    result = mpod_channel.getCurrent()
    print("getCurrent() returns \"{0}\"".format(result))

    result = mpod_channel.getCurrentFallRate()
    print("getCurrentFallRate() returns \"{0}\"".format(result))

    result = mpod_channel.getCurrentRiseRate()
    print("getCurrentRiseRate() returns \"{0}\"".format(result))

    result = mpod_channel.getSupervisionBehavior()
    print("getSupervisionBehavior() returns \"{0}\"".format(result))

    result = mpod_channel.getSupervisionMaxCurrent()
    print("getSupervisionMaxCurrent() returns \"{0}\"".format(result))

    result = mpod_channel.getSupervisionMaxTerminalVoltage()
    print("getSupervisionMaxTerminalVoltage() returns \"{0}\"".format(result))

    result = mpod_channel.getSupervisionMinSenseVoltage()
    print("getSupervisionMinSenseVoltage() returns \"{0}\"".format(result))

    result = mpod_channel.getSwitch()
    print("getSwitch() returns \"{0}\"".format(result))

    result = mpod_channel.getTripTime()
    print("getTripTime() returns \"{0}\"".format(result))

    result = mpod_channel.getUserConfig()
    print("getUserConfig() returns \"{0}\"".format(result))

    result = mpod_channel.getVoltage()
    print("getVoltage() returns \"{0}\"".format(result))

    result = mpod_channel.getVoltageFallRate()
    print("getVoltageFallRate() returns \"{0}\"".format(result))

    result = mpod_channel.getVoltageRiseRate()
    print("getVoltageRiseRate() returns \"{0}\"".format(result))


def test_module(mpod_module) :

    nb_of_channels = mpod_module.getNumberOfChannels()
    print("getNumberOfChannels() returns \"{0}\"".format(nb_of_channels))

    for channel_index in range(0, nb_of_channels) :
        mpod_channel = mpod_module.getChannel(channel_index)
        print("### testing channel function for channel \"{0}\"".format(mpod_channel))
        test_channel_functions(mpod_channel)





thread_polling_interval_ms = 1000.0
wait_delay = 3 * thread_polling_interval_ms * 0.001

# Here we build the relative path to the python module.
# The path might have to be adjusted if the example is run outside the 
# Analog Flavor software installation tree.
mpod_snmp = import_mpod_snmp_module.import_mpod_snmp_module()

# the IP address, port and snmp community can be specified in the first argument as
#           crateConnect("192.168.0.101:161:guru", ...
#       or in the first 3 arguments as
#           crateConnect("192.168.0.101", 161, "guru", ...
# the polling intervals are optional arguments
if (len(sys.argv) >= 2) :
    mpod_crate = mpod_snmp.crateConnect(sys.argv[1], 0, "", 0.5 * thread_polling_interval_ms, thread_polling_interval_ms, 2.0 * thread_polling_interval_ms);
else :
    print("Please specify an MPOD connection on the comnmand line (ex. \"192.168.0.100:161:guru\").")
    default_crate_ip = import_mpod_snmp_module.get_default_crate_ip()
    mpod_crate = mpod_snmp.crateConnect(default_crate_ip, 0, "")

print("newly created crate  == {0}".format(mpod_crate))

# check that the connection has been established 
if (mpod_crate.checkConnectionIsOk()) :

    # wait to make sure all data has been read to the buffer 
    time.sleep(wait_delay)

    all_slots = mpod_crate.getOccupiedSlots()
    print("getOccupiedSlots() returns \"{0}\"".format(all_slots))

    for slot_index in all_slots:
        mpod_module = mpod_crate.getModule(slot_index)
        print("### testing module \"{0}\"".format(mpod_module))
        test_module(mpod_module)

else :
    # print error messages 
    print("checkConnectionIsOk() returns false.");
    int_value = mpod_crate.getConnectionStatus()
    print("getConnectionStatus() returns {0}.".format(int_value))
    string_value = mpod_crate.getConnectionStatusAsString()
    print("getConnectionStatusAsString() returns {0}.".format(string_value))
    string_value = mpod_crate.getErrorString()
    print("getErrorString() returns {0}.".format(string_value))

exit(0)


